@@ -89,7 +89,10 @@ KQuickStyleItem::KQuickStyleItem(QQuickItem *parent)
     m_textureHeight(0),
     m_lastFocusReason(Qt::NoFocusReason)
 {
-    if (!qApp->style()) {
+    // There is no styleChanged signal and QApplication sends QEvent::StyleChange only to all QWidgets
+    if (qApp->style()) {
+        connect(qApp->style(), &QObject::destroyed, this, &KQuickStyleItem::styleChanged);
+    } else {
         KSharedConfig::Ptr kdeglobals = KSharedConfig::openConfig();
         KConfigGroup cg(kdeglobals, "KDE");
         auto style = s_style;
@@ -102,7 +105,6 @@ KQuickStyleItem::KQuickStyleItem(QQuickItem *parent)
     m_font = qApp->font();
     setFlag(QQuickItem::ItemHasContents, true);
     setSmooth(false);
-    qmlRegisterType<KQuickPadding>();
 
     connect(this, &KQuickStyleItem::visibleChanged, this, &KQuickStyleItem::updateItem);
     connect(this, &KQuickStyleItem::widthChanged, this, &KQuickStyleItem::updateItem);
@@ -137,6 +139,8 @@ KQuickStyleItem::KQuickStyleItem(QQuickItem *parent)
 
     connect(this, &KQuickStyleItem::heightChanged, this, &KQuickStyleItem::updateBaselineOffset);
     connect(this, &KQuickStyleItem::contentHeightChanged, this, &KQuickStyleItem::updateBaselineOffset);
+
+    connect(qApp, &QApplication::fontChanged, this, &KQuickStyleItem::updateSizeHint, Qt::QueuedConnection);
 }
 
 KQuickStyleItem::~KQuickStyleItem()
@@ -1686,9 +1690,6 @@ bool KQuickStyleItem::event(QEvent *ev)
             polish();
         }
         return true;
-    } else if (ev->type() == QEvent::StyleChange) {
-        if (m_itemType == ScrollBar)
-            initStyleOption();
     }
     return QQuickItem::event(ev);
 }
@@ -1818,6 +1819,20 @@ bool KQuickStyleItem::eventFilter(QObject *watched, QEvent *event)
     return QQuickItem::eventFilter(watched, event);
 }
 
+void KQuickStyleItem::styleChanged()
+{
+    if (!qApp->style() || QApplication::closingDown()) {
+        return;
+    }
+
+    Q_ASSERT(qApp->style() != sender());
+
+    connect(qApp->style(), &QObject::destroyed, this, &KQuickStyleItem::styleChanged);
+
+    updateSizeHint();
+    updateItem();
+}
+
 QPixmap QQuickTableRowImageProvider1::requestPixmap(const QString &id, QSize *size, const QSize &requestedSize)
 {
     Q_UNUSED (requestedSize);
